/*
 * Copyright (c) 2014 Macrofocus GmbH. All Rights Reserved.
 */
package com.macrofocus.molap.adapter;

import com.macrofocus.data.table.AbstractTableModel;
import com.macrofocus.molap.aggregates.cube.Dimensions;
import com.macrofocus.molap.aggregates.cube.Group;
import com.macrofocus.molap.aggregates.cube.GroupFactory;
import com.macrofocus.molap.aggregates.query.Query;
import com.macrofocus.molap.aggregates.query.QueryListener;

import java.util.HashMap;
import java.util.Map;

public class AggregatesTableModel extends AbstractTableModel {
    private Query query;

    private Dimensions dimensions;

    private Map<Group,Integer> index;

    private final QueryListener queryListener = new QueryListener() {
        @Override
        public void queryChanged() {
            index = null;
            fireTableStructureChanged();
        }
    };

    public AggregatesTableModel(Query query) {
        this.query = query;

        query.addWeakQueryListener(queryListener);
    }

    @Override
    public String getColumnName(int columnIndex) {
        if(columnIndex < getDimensions().path.length) {
            return getDimensions().path[columnIndex].toString();
        }
        columnIndex -= getDimensions().path.length;
        return query.getAggregations()[columnIndex].getName();
    }

    @Override
    public Class<?> getColumnClass(int columnIndex) {
        if(columnIndex < getDimensions().path.length) {
            return query.getCuboid().getCube().getDataFrame().getColumnClass(getDimensions().path[columnIndex]);
        }
        columnIndex -= getDimensions().path.length;
        return query.getAggregations()[columnIndex].getType();
    }

    @Override
    public int getRowCount() {
        return query.getGroups().size();
    }

    @Override
    public int getColumnCount() {
        return getDimensions().path.length + query.getAggregations().length;
    }

    @Override
    public Object getValueAt(int rowIndex, int columnIndex) {
        final Group path = query.getGroups().get(rowIndex);
        assert path != null;
        if(columnIndex < getDimensions().path.length) {
            if(dimensions != null) {
                final Object column = getDimensions().path[columnIndex];
                columnIndex = query.getDimensions().getIndex(column);
            }
            return path.getPath(columnIndex);
        }
        columnIndex -= getDimensions().path.length;
        return query.getValue(path, query.getAggregations()[columnIndex]);
    }

    public void setDimensions(Dimensions dimensions) {
        if(this.dimensions != dimensions) {
            this.dimensions = dimensions;

            this.index = null;

            fireTableStructureChanged();
        }
    }

    public Dimensions getDimensions() {
        return dimensions != null? dimensions : query.getDimensions();
    }

    public void setQuery(Query query) {
        if(this.query != query) {
            if(this.query != null) {
                this.query.removeQueryListener(queryListener);
            }

            this.query = query;
            this.index = null;

            if(this.query != null) {
                this.query.addWeakQueryListener(queryListener);
            }

            fireTableStructureChanged();
        }
    }

    public Integer getGroupIndex(Group group) {
        if(index == null) {
            index = new HashMap<Group, Integer>();
            for(int r = 0; r < getRowCount(); r++) {
                Group g = getGroup(r);
                if(!index.containsKey(g)) {
                    index.put(g, r);
                } else {
                    System.err.println("Not unique " + g);
                }
            }
        }
        return index.get(group);
    }

    public Group getGroup(int row) {
        final Object[] path = getDimensions().path;
        Object[] values = new Object[path.length];
        for (int i = 0; i < path.length; i++) {
            values[i] = getValueAt(row, i);
        }
        return GroupFactory.create(values);
    }
}
