/*
 * Copyright (c) 2014 Macrofocus GmbH. All Rights Reserved.
 */
package com.macrofocus.molap.aggregates;

import com.macrofocus.molap.aggregates.aggregation.Aggregation;
import com.macrofocus.molap.aggregates.cube.Group;
import com.macrofocus.molap.aggregates.cuboid.Cuboid;
import com.macrofocus.molap.dataframe.DataFrame;

/**
 * A data frame specialized for aggregating values. For obtaining an instance of such an interface,
 * see {@link com.macrofocus.molap.dataframe.DataFrame#aggregate(com.macrofocus.molap.aggregates.aggregation.Aggregation...)}.
 *
 * @param <C> the original type of columns
 */
public interface AggregateDataFrame<C> extends DataFrame<Group,String,Object> {
    /**
     * Returns the cuboid used to aggregate the values.
     *
     * @return the cuboid
     */
    Cuboid getCuboid();

    /**
     * Returns a new data frame that group the rows by values of the specified columns.
     *
     * @param columns the columns
     * @return a new data frame aggregating the values according to the new criteria
     */
    AggregateDataFrame<C> drillDown(C... columns);

    /**
     * Returns a new data frame with the last group by level removed, i.e. one level up.
     *
     * @return a new data frame aggregating the values according to the new criteria
     */
    AggregateDataFrame<C> drillUp();

    /**
     * Returns a new data frame that change the order of the groups according to the
     * specified aggregation methods.
     *
     * @param aggregations the aggregation methods
     * @return a new data frame
     */
    AggregateDataFrame<C> order(Aggregation... aggregations);

    /**
     * Returns a new data frame that pivot the specified aggregation method into column values.
     *
     * @param aggregation the aggregation to pivot
     * @return a new data frame
     */
    AggregateDataFrame<C> pivot(Aggregation aggregation);

    /**
     * Combine the values from multiple cuboids into that data frame.
     *
     * @param cuboids the cuboids
     * @return a new data frame
     */
    AggregateDataFrame<C> on(Cuboid... cuboids);
}
