/*
 * Copyright (c) 2014 Macrofocus GmbH. All Rights Reserved.
 */
package com.macrofocus.molap.aggregates.aggregation;

import com.macrofocus.molap.aggregates.cube.Group;
import com.macrofocus.molap.aggregates.cuboid.Cuboid;

public abstract class AbstractAggregation implements Aggregation {
    @Override
    public Aggregation as(final String name) {
        return new AbstractAggregation() {
            @Override
            public String getName() {
                return name;
            }

            @Override
            public Class getType() {
                return AbstractAggregation.this.getType();
            }

            @Override
            public Object compute(Cuboid cuboid, Group group) {
                return AbstractAggregation.this.compute(cuboid, group);
            }
        };
    }

    @Override
    public String getName() {
        return toString();
    }

    @Override
    public Aggregation dividedBy(Aggregation aggregation) {
        return new DividedByAggregation(this, aggregation);
    }

    private static class DividedByAggregation extends AbstractAggregation {
        private final Aggregation a1;
        private final Aggregation a2;

        private DividedByAggregation(Aggregation a1, Aggregation a2) {
            this.a1 = a1;
            this.a2 = a2;
        }

        @Override
        public Class getType() {
            return Double.class;
        }

        @Override
        public Object compute(Cuboid cuboid, Group group) {
            final Object v1 = a1.compute(cuboid, group);
            final Object v2 = a2.compute(cuboid, group);
            if(v1 != null && v2 != null && v1 instanceof Number && v2 instanceof Number && ((Number) v2).doubleValue() != 0) {
                return ((Number) v1).doubleValue() / ((Number)v2).doubleValue();
            } else {
                return null;
            }
        }

        @Override
        public String toString() {
            return a1.getName() + "/" + a2.getName();
        }
    }

    @Override
    public Aggregation stdDevBy(Aggregation aggregation) {
        return new StdDevByAggregation(this, aggregation);
    }

    private static class StdDevByAggregation extends AbstractAggregation {
        private final Aggregation a1;
        private final Aggregation a2;

        private StdDevByAggregation(Aggregation a1, Aggregation a2) {
            this.a1 = a1;
            this.a2 = a2;
        }

        @Override
        public Class getType() {
            return Double.class;
        }

        @Override
        public Object compute(Cuboid cuboid, Group group) {
            final Object v1 = a1.compute(cuboid, group);
            final Object v2 = a2.compute(cuboid, group);
            if(v1 != null && v2 != null && v1 instanceof Number && v2 instanceof Number && ((Number) v2).doubleValue() != 0) {
                return ((Number) v1).doubleValue() / ((Number)v2).doubleValue();
            } else {
                return null;
            }
        }

        @Override
        public String toString() {
            return a1.getName() + "/" + a2.getName();
        }
    }
}
