/*
 * Copyright (c) 2014 Macrofocus GmbH. All Rights Reserved.
 */
package com.macrofocus.molap.aggregates.aggregation;

import com.macrofocus.molap.aggregates.cube.DistributiveStatistics;
import com.macrofocus.molap.aggregates.cube.Group;
import com.macrofocus.molap.aggregates.cuboid.Cuboid;
import com.macrofocus.molap.series.Series;

public class StdDevAggregation extends AbstractAggregation {
    private final Series column;
    private final Series sum;
    private final Series count;

    public StdDevAggregation(Series column) {
        this.column = column;
        this.sum = null;
        this.count = null;
    }

    public StdDevAggregation(Series uncorrectedSumOfSquares, Series sum, Series count) {
        this.column = uncorrectedSumOfSquares;
        this.sum = sum;
        this.count = count;
    }

    @Override
    public Class getType() {
        return Double.class;
    }

    @Override
    public Object compute(Cuboid cuboid, Group group) {
        final DistributiveStatistics d = cuboid.getDistributiveStatistics(group, column);
        if(count != null) {
            final DistributiveStatistics p = cuboid.getDistributiveStatistics(group, count);
            final DistributiveStatistics s = cuboid.getDistributiveStatistics(group, sum);
            final Double mean = s.getSum() / p.getSum();
            double variance = d.getSum() / p.getSum() - (mean * mean);
            return Math.sqrt(variance);
        } else {
            return d.getStandardDeviation();
        }
    }

    @Override
    public String toString() {
        return "StdDev(" + column.getName() + ")";
    }
}
