/*
 * Copyright (c) 2014 Macrofocus GmbH. All Rights Reserved.
 */
package com.macrofocus.molap.aggregates.aggregation;

import com.macrofocus.molap.aggregates.cube.DistributiveStatistics;
import com.macrofocus.molap.aggregates.cube.Group;
import com.macrofocus.molap.aggregates.cuboid.Cuboid;
import com.macrofocus.molap.series.Series;

public class VarianceAggregation extends AbstractAggregation {
    private final Series column;
    private final Series population;

    public VarianceAggregation(Series column) {
        this.column = column;
        this.population = null;
    }

    public VarianceAggregation(Series column, Series population) {
        this.column = column;
        this.population = population;
    }

    @Override
    public Class getType() {
        return Double.class;
    }

    @Override
    public Object compute(Cuboid cuboid, Group group) {
        final DistributiveStatistics d = cuboid.getDistributiveStatistics(group, column);
        if(population != null) {
            final DistributiveStatistics p = cuboid.getDistributiveStatistics(group, population);
            return d.getUncorrectedSumOfSquares() / p.getCount() - (d.getMean() * d.getMean());
        } else {
            return d.getVariance();
        }
    }

    @Override
    public String toString() {
        return "Variance(" + column.getName() + ")";
    }
}
