/*
 * Copyright (c) 2014 Macrofocus GmbH. All Rights Reserved.
 */
package com.macrofocus.molap.aggregates.cube;

import java.util.Arrays;

public class ArrayGroup implements Group {
    private static final boolean FLYWEIGHT = false;

    private static final FlyweightSet<Group> cache = FLYWEIGHT ? new FlyweightSet<Group>() {
        // ToDo: reimplement;
//        @Override
//        protected int hashCode(Group obj) {
//            return Arrays.hashCode((obj).path);
//        }
//
//        @Override
//        protected boolean equals(Group a, Group b) {
//            if (a == b) return true;
//
//            // Probably incorrect - comparing Object[] arrays with Arrays.equals
//            return Arrays.equals(a.path, b.path);
//
//        }
    } : null;

    static Group get(Object... path) {
        if(FLYWEIGHT) {
            return cache.put(new ArrayGroup(path));
        } else {
            return new ArrayGroup(path);
        }
    }

    private static Group getDrillDown(Object last, Object... path) {
        Object[] newPath = new Object[path.length + 1];
        System.arraycopy(path, 0, newPath, 0, path.length);
        newPath[newPath.length - 1] = last;
        return get(newPath);
    }

    private static Group getDrillUp(Object... path) {
        Object[] newPath = new Object[path.length - 1];
        System.arraycopy(path, 0, newPath, 0, newPath.length);
        return get(newPath);
    }

    private final Object[] path;

    protected ArrayGroup(Object... path) {
        this.path = path;
    }

    @Override
    public Group drillUp() {
        if(this.path.length > 0) {
            return getDrillUp(this.path);
        } else {
            return null;
        }
    }

    @Override
    public Group drillDown(Object valueAt) {
        return getDrillDown(valueAt, this.path);
    }

    @Override
    public Object getLast() {
        return path[path.length - 1];
    }

    @Override
    public Object getPath(int i) {
        return path[i];
    }

    @Override
    public Object[] getPath() {
        return path;
    }

    @Override
    public int getPathLength() {
        return path.length;
    }

    @Override
    public boolean equals(Object o) {
        if(FLYWEIGHT) {
            return this == o;
        } else {
            if (this == o) return true;
            if (!(o instanceof ArrayGroup)) return false;

            ArrayGroup path1 = (ArrayGroup) o;

            // Probably incorrect - comparing Object[] arrays with Arrays.equals
            return Arrays.equals(path, path1.path);

        }
    }

    @Override
    public int hashCode() {
        if(FLYWEIGHT) {
            return super.hashCode();
        } else {
            return Arrays.hashCode(path);
        }
    }

//    @Override
//    public int compareTo(Object o) {
//        if (this == o) return 0;
//        if (!(o instanceof Group)) return -1;
//
//        Group group = (Group) o;
//
//        int length = Math.min(group.path.length, path.length);
//
//        for (int i=0; i<length; i++) {
//            Comparable o1 = (Comparable)path[i];
//            Comparable o2 = (Comparable)group.path[i];
//            if(o1 != null) {
//                if(o2 != null) {
//                    int equals = o1.compareTo(o2);
//                    if(equals != 0) {
//                        return o1.compareTo(o2);
//                    }
//                } else {
//                    return 1;
//                }
//            } else {
//                if(o2 != null) {
//                    return -1;
//                }
//            }
//        }
//
//        return group.path.length < path.length ? 1: 0;
//    }

    @Override
    public String toString() {
        return "ArrayGroup{" +
                "path=" + Arrays.toString(path) +
                '}';
    }

    @Override
    public boolean startsWith(Group group) {
        if (this==group)
            return true;
        if (group==null)
            return false;

        int length = this.path.length;
        final Object[] otherPath = group.getPath();
        if (otherPath.length >= length)
            return false;

        for (int i = 0; i < otherPath.length; i++) {
            Object o1 = path[i];
            Object o2 = otherPath[i];
            if (!(o1 == null ? o2 == null : o1.equals(o2)))
                return false;
        }

        return true;
    }
}
