/*
 * Copyright (c) 2014 Macrofocus GmbH. All Rights Reserved.
 */
package com.macrofocus.molap.aggregates.cube;

import com.macrofocus.molap.aggregates.aggregation.*;
import com.macrofocus.molap.aggregates.cuboid.Cuboid;
import com.macrofocus.molap.aggregates.query.Query;
import com.macrofocus.molap.aggregates.query.Structure;

/**
 * Models a multidimensional dataset, where that data are aggregated across an arbitrary number of dimensions.
 * It can also be seen as a lattice of cuboids.
 */
public interface Cube {
    com.macrofocus.molap.dataframe.DataFrame getDataFrame();

    Cuboid getApexCuboid();

    Dimensions getRoot();

    Dice getDice();

    Structure getStructure();

    Cuboid getCuboid(Dimensions dimensions, Dice dice, Structure structure);

    Cuboid getChildCuboid(Cuboid cuboid);

    Query query(Aggregation... aggregations);

    SumAggregation getSum(String name);

    CountAggregation getCount(String name);

    MinAggregation getMin(String name);

    MaxAggregation getMax(String name);

    MeanAggregation getMean(String name);

    VarianceAggregation getVariance(String name);

    VarianceAggregation getVariance(String name, String population);

    StdDevAggregation getStdDev(String name);

    CountDistinctAggregation getCountDistinct(String name);
}
