/*
 * Copyright (c) 2014 Macrofocus GmbH. All Rights Reserved.
 */
package com.macrofocus.molap.aggregates.cube;

import java.util.Arrays;
import java.util.Set;

public class Dice {
    private static final boolean FLYWEIGHT = false;

    private static final FlyweightSet<Dice> cache = FLYWEIGHT ? new FlyweightSet<Dice>() : null;

    private static Dice get(Set<Object>... path) {
        if(FLYWEIGHT) {
            return cache.put(new Dice(path));
        } else {
            return new Dice(path);
        }
    }

    public final Set<Object>[] path;

    public Dice(Set<Object>... path) {
        this.path = path;
    }

    public static Dice create(Set<Object>... path) {
        return get(path);
    }

    public Dice drillDown(Set<Object>... columns) {
        Set<Object>[] path = new Set[this.path.length + columns.length];
        System.arraycopy(this.path, 0, path, 0, this.path.length);
        System.arraycopy(columns, 0, path, this.path.length, columns.length);
        return Dice.create(path);
    }

    public Dice drillUp() {
        assert this.path.length > 0: this;

        if(this.path.length > 0) {
            Set<Object>[] path = new Set[this.path.length - 1];
            System.arraycopy(this.path, 0, path, 0, path.length);
            return Dice.create(path);
        } else {
            return null;
        }
    }

    public Set<Object> getLast() {
        return path[path.length - 1];
    }

    public Dice changeLast(Set<Object> valuesSets) {
        return drillUp().drillDown(valuesSets);
    }

    @Override
    public boolean equals(Object o) {
        if(FLYWEIGHT) {
            return this == o;
        } else {
            if (this == o) return true;
            if (!(o instanceof Dice)) return false;

            Dice path1 = (Dice) o;

            // Probably incorrect - comparing Object[] arrays with Arrays.equals
            return Arrays.equals(path, path1.path);

        }
    }

    @Override
    public int hashCode() {
        if(FLYWEIGHT) {
            return super.hashCode();
        } else {
            return Arrays.hashCode(path);
        }
    }

    @Override
    public String toString() {
        return "Dice{" +
                "path=" + Arrays.toString(path) +
                '}';
    }
}