/*
 * Copyright (c) 2014 Macrofocus GmbH. All Rights Reserved.
 */
package com.macrofocus.molap.aggregates.cube;

import com.macrofocus.molap.series.Series;

public class DistributiveStatistics {
    private long count;
    private long countOfMissing;
    private double sum;
    private double uncorrectedSumOfSquares;
    private Number minimum;
    private Number maximum;

    public DistributiveStatistics() {
        count = 0;
        countOfMissing = 0;
        sum = 0;
        uncorrectedSumOfSquares = 0;
        minimum = null;
        maximum = null;
    }

    public <R> DistributiveStatistics(Series<R, ?> series) {
        this(series, series.keys());
    }

    public <R> DistributiveStatistics(Series<R, ?> series, Iterable<R> keys) {
        this();
        for (R row : keys) {
            assert row != null;

            Object value = series.get(row);
            if(value != null && value instanceof Number) {
                final double v = ((Number) value).doubleValue();
                sum += v;
                uncorrectedSumOfSquares += v * v;
                count++;
                if(minimum == null || v < minimum.doubleValue()) {
                    minimum = (Number)value;
                }
                if(maximum == null || v > maximum.doubleValue()) {
                    maximum = (Number)value;
                }
            } else {
                countOfMissing++;
            }
        }
    }

    public void add(DistributiveStatistics distributiveStatistics) {
        count += distributiveStatistics.getCount();
        countOfMissing += distributiveStatistics.getCountOfMissing();
        sum += distributiveStatistics.getSum();
        uncorrectedSumOfSquares += distributiveStatistics.getUncorrectedSumOfSquares();
        if(minimum == null || distributiveStatistics.getMinimum().doubleValue() < minimum.doubleValue()) {
            minimum = distributiveStatistics.getMinimum();
        }
        if(maximum == null || distributiveStatistics.getMaximum().doubleValue() > maximum.doubleValue()) {
            maximum = distributiveStatistics.getMaximum();
        }
    }

    //

    public long getCount() {
        return count;
    }

    public double getSum() {
        return sum;
    }

    public double getUncorrectedSumOfSquares() {
        return uncorrectedSumOfSquares;
    }

    public long getCountOfMissing() {
        return countOfMissing;
    }

    public Number getMinimum() {
        return minimum;
    }

    public Number getMaximum() {
        return maximum;
    }

    public Double getMean() {
        if(count > 0) {
            return getSum() / getCount();
        } else {
            return null;
        }
    }

    public Double getRange() {
        return getMaximum().doubleValue() - getMinimum().doubleValue();
    }

    public Double getCorrectedSumOfSquares() {
        return null;
    }

    public Double getVariance() {
        final Double mean = getMean();
        if(mean != null) {
            return getUncorrectedSumOfSquares() / getCount() - (mean * mean);
        } else {
            return null;
        }
    }

    public Double getStandardDeviation() {
        final Double variance = getVariance();
        if(variance != null) {
            return Math.sqrt(variance);
        } else {
            return null;
        }
    }

    public Double getStandardErrorOfMean() {
        return null;
    }

    public Double getCoefficientOfVariance() {
        return null;
    }

    public Double getTValue() {
        return null;
    }

    public Double getProbabilityOfGreaterAbsoluteValue() {
        return null;
    }

    public Double getLowerConfidenceLimit() {
        return null;
    }

    public Double getUpperConfidenceLimit() {
        return null;
    }

    @Override
    public String toString() {
        return "DistributiveStatistics{" +
                "count=" + count +
                ", countOfMissing=" + countOfMissing +
                ", sum=" + sum +
                ", uncorrectedSumOfSquares=" + uncorrectedSumOfSquares +
                ", minimum=" + minimum +
                ", maximum=" + maximum +
                ", mean=" + getMean() +
                ", variance=" + getVariance() +
                ", stdDev=" + getStandardDeviation() +
                '}';
    }
}
