/*
 * Copyright (c) 2014 Macrofocus GmbH. All Rights Reserved.
 */
package com.macrofocus.molap.aggregates.cube;

import java.util.Arrays;
import java.util.HashMap;
import java.util.Map;

public class LinkedGroup implements Group {
    private final int level;
    private final LinkedGroup parent;
    private final Object value;

    private Map<Object,LinkedGroup> children;

    public LinkedGroup() {
        level = 0;
        parent = null;
        value = null;
    }

    public LinkedGroup(LinkedGroup parent, Object value) {
        assert parent != null;

        this.parent = parent;
        this.value = value;
        this.level = parent.level + 1;
    }

    @Override
    public Group drillUp() {
        return parent;
    }

    @Override
    public Group drillDown(Object valueAt) {
        final Group child;
        if(children == null) {
            children = new HashMap<Object, LinkedGroup>();
            child = null;
        } else {
            child = children.get(valueAt);
        }
        if(child == null) {
            final LinkedGroup group = new LinkedGroup(this, valueAt);
            children.put(valueAt, group);
            return group;
        } else {
            return child;
        }
    }

    @Override
    public Object getLast() {
        return value;
    }

    @Override
    public Object getPath(int level) {
        LinkedGroup current = this;
        for (int i = this.level - 1; i > level; i--) {
            current = current.parent;
        }
        return current.value;
    }

    @Override
    public Object[] getPath() {
        Object[] path = new Object[level];

        LinkedGroup current = this;
        for (int i = level - 1; i >= 0; i--) {
            path[i] = current.value;
            current = current.parent;
        }

        return path;
    }

    @Override
    public int getPathLength() {
        return level;
    }

    @Override
    public boolean startsWith(Group group) {
        if (this==group)
            return true;
        if (group==null)
            return false;

        Object[] thisPath = getPath();

        int length = thisPath.length;
        final Object[] otherPath = group.getPath();
        if (otherPath.length >= length)
            return false;

        for (int i = 0; i < otherPath.length; i++) {
            Object o1 = thisPath[i];
            Object o2 = otherPath[i];
            if (!(o1 == null ? o2 == null : o1.equals(o2)))
                return false;
        }

        return true;
    }

    @Override
    public String toString() {
        return "LinkedGroup{" +
                "path=" + Arrays.toString(getPath()) +
                '}';
    }
}
