/*
 * Copyright (c) 2014 Macrofocus GmbH. All Rights Reserved.
 */
package com.macrofocus.molap.aggregates.cuboid;

import com.macrofocus.molap.aggregates.cube.*;
import com.macrofocus.molap.dataframe.DataFrameEvent;
import com.macrofocus.molap.dataframe.DataFrameListener;

import java.util.Arrays;
import java.util.Collections;
import java.util.List;
import java.util.Set;

/**
 * The top-most cuboid, which holds the highest-level of summarization.
 */
public class ApexCuboid extends AbstractCuboid {
    private final Dimensions dimensions;

    private final Group group = GroupFactory.createRoot();
    private final List<Group> groups = Collections.singletonList(group);

    private final DataFrameListener listener = new DataFrameListener() {
        @Override
        public void dataFrameChanged(DataFrameEvent event) {
            setDirty(true);
        }
    };

    public ApexCuboid(InMemoryCube cube, Dimensions dimensions) {
        super(cube);
        this.dimensions = dimensions;

        cube.getDataFrame().addWeakDataFrameListener(listener);
    }

    @Override
    public Cuboid drillDown(Object... columns) {
        final Dimensions dimensions = cube.getRoot().drillDown(columns);
        Set[] diceValues = new Set[columns.length];
        Boolean[] structureValues = new Boolean[columns.length];
        Arrays.fill(structureValues, Boolean.FALSE);
        return cube.getCuboid(dimensions, cube.getDice().drillDown(diceValues), cube.getStructure().drillDown(structureValues));
    }

    @Override
    public Cuboid drillUp() {
        assert false;

        return null;
    }

    @Override
    public Cuboid slice(Object values) {
        return null;
    }

    @Override
    public Cuboid dice(Set<Object> valuesSets) {
        return null;
    }

    @Override
    public Cuboid collapse() {
        return null;
    }

    @Override
    public Cube getCube() {
        return cube;
    }

    @Override
    public Dimensions getDimensions() {
        return dimensions;
    }

    @Override
    public Iterable<Group> getGroups() {
        return groups;
    }

    @Override
    public Iterable<Group> getGroupsStartingWith(Group group) {
        return null;
    }

    @Override
    public int getGroupsStartingWithCount(Group group) {
        return 0;
    }

    @Override
    public int getGroupsCount() {
        return groups.size();
    }

    @Override
    public Set<Object> getValues() {
        return Collections.emptySet();
    }

    @Override
    public Iterable<Object> getRows(Group group) {
        assert group == this.group;

        return cube.getDataFrame().rows();
    }
}
