/*
 * Copyright (c) 2014 Macrofocus GmbH. All Rights Reserved.
 */
package com.macrofocus.molap.aggregates.query;

import com.macrofocus.molap.aggregates.aggregation.Aggregation;
import com.macrofocus.molap.aggregates.cube.ArrayGroup;
import com.macrofocus.molap.aggregates.cube.Dimensions;
import com.macrofocus.molap.aggregates.cube.Group;
import com.macrofocus.molap.aggregates.cuboid.Cuboid;

import java.util.*;

public class CuboidsQuery extends AbstractQuery {
    private final Cuboid[] cuboids;
    private final Aggregation[] aggregations;

    private List<Group> groups;
    private Dimensions dimensions;

    public CuboidsQuery(Cuboid[] cuboids, Aggregation... aggregations) {
        assert cuboids != null;

        this.cuboids = cuboids;
        this.aggregations = aggregations;
    }

    private void run() {
        Set<Object> columns = new HashSet<Object>();
        for (Cuboid cuboid : cuboids) {
            Dimensions d = cuboid.getDimensions();
            Collections.addAll(columns, d.path);
        }
        dimensions = Dimensions.create(columns.toArray(new Object[columns.size()]));

        groups = new ArrayList<Group>();
        for (Cuboid cuboid : cuboids) {
            for (Group group : cuboid.getGroups()) {
                groups.add(new CuboidGroup(cuboid, dimensions, group));
            }
        }
        setDirty(false);
    }

    @Override
    public Object getValue(Group path, Aggregation aggregation) {
        return aggregation.compute(((CuboidGroup)path).getCuboid(), ((CuboidGroup)path).getOriginalGroup());
    }

    @Override
    public Dimensions getDimensions() {
        if(isDirty()) {
            run();
        }
        return dimensions;
    }

    @Override
    public Aggregation[] getAggregations() {
        return aggregations;
    }

    @Override
    public List<Group> getGroups() {
        if(isDirty()) {
            run();
        }

        return groups;
    }

    @Override
    public Cuboid getCuboid() {
        return null;
    }

    @Override
    public void setDice(Set<Object> valuesSets) {
        throw new UnsupportedOperationException();
    }

    private static class CuboidGroup extends ArrayGroup {
        private final Cuboid cuboid;
        private final Dimensions dimensions;
        private final Group group;

        private CuboidGroup(Cuboid cuboid, Dimensions dimensions, Group group) {
            super(group.getPath());
            this.cuboid = cuboid;
            this.dimensions = dimensions;
            this.group = group;
        }

        public Group getOriginalGroup() {
            return group;
        }

        @Override
        public Object[] getPath() {
            Object[] path = new Object[dimensions.path.length];

            for (int i = 0; i < dimensions.path.length; i++) {
                Object column = dimensions.path[i];

                // Find column index
                final Integer index = cuboid.getDimensions().getIndex(column);
                if(index != null) {
                    path[i] = group.getPath(index);
                } else {
                    path[i] = Cuboid.ALL;
                }
            }

            return path;
        }

        @Override
        public Object getPath(int i) {
            return getPath()[i];
        }

        public Cuboid getCuboid() {
            return cuboid;
        }
    }
}
