/*
 * Copyright (c) 2014 Macrofocus GmbH. All Rights Reserved.
 */
package com.macrofocus.molap.aggregates.query;

import com.macrofocus.colormap.SimpleColorMap;
import com.macrofocus.molap.aggregates.aggregation.Aggregation;
import com.macrofocus.molap.aggregates.cube.Dimensions;
import com.macrofocus.molap.aggregates.cube.Group;
import com.macrofocus.molap.aggregates.cuboid.Cuboid;

import java.util.*;

public class OrderQuery extends AbstractQuery {
    private final Query query;
    private final Aggregation[] aggregations;

    private List<Group> groups;

    private final Comparator universalComparator = new SimpleColorMap.UniversalComparator();

    final QueryListener queryListener = new QueryListener() {
        @Override
        public void queryChanged() {
            setDirty(true);
            notifyQueryChanged();
        }
    };

    public OrderQuery(Query query, Aggregation[] aggregations) {
        this.query = query;
        this.aggregations = aggregations;

        query.addWeakQueryListener(queryListener);
    }

    private void run() {
        List<Group> groups = new ArrayList<Group>();
        for (Group group : query.getGroups()) {
            groups.add(group);
        }

        Collections.sort(groups, new Comparator<Group>() {
            @Override
            public int compare(Group o1, Group o2) {
                return compare(query, o1, o2);
            }

            public int compare(Query query, Group o1, Group o2) {
                int length = Math.min(o1.getPathLength(), o2.getPathLength());

                if(length > 1) {
                    int c = compare(query.drillUp(), o1.drillUp(), o2.drillUp());
                    if(c != 0) {
                        return c;
                    }
                }

                for (Aggregation aggregation : aggregations) {
                    Object v1 = query.getValue(o1, aggregation);
                    Object v2 = query.getValue(o2, aggregation);

                    final int compare = universalComparator.compare(v1, v2);
                    if(compare != 0) {
                        return compare;
                    }
                }
                final int x = o1.getPathLength();
                final int y = o2.getPathLength();
                return  (x < y) ? -1 : ((x == y) ? 0 : 1);
            }
        });
        this.groups = groups;

        setDirty(false);
    }

    @Override
    public Object getValue(Group path, Aggregation aggregation) {
        return query.getValue(path, aggregation);
    }

    @Override
    public Dimensions getDimensions() {
        return query.getDimensions();
    }

    @Override
    public Aggregation[] getAggregations() {
        return query.getAggregations();
    }

    @Override
    public List<Group> getGroups() {
        if(isDirty()) {
            run();
        }
        return groups;
    }

    @Override
    public Cuboid getCuboid() {
        return query.getCuboid();
    }

    @Override
    public void setDice(Set<Object> valuesSets) {
        query.setDice(valuesSets);
    }
}
