/*
 * Copyright (c) 2014 Macrofocus GmbH. All Rights Reserved.
 */
package com.macrofocus.molap.aggregates.query;

import com.macrofocus.molap.adapter.AggregatesTableModel;
import com.macrofocus.molap.aggregates.aggregation.Aggregation;
import com.macrofocus.molap.aggregates.cube.Dimensions;
import com.macrofocus.molap.aggregates.cube.Group;
import com.macrofocus.molap.aggregates.cuboid.Cuboid;
import com.macrofocus.molap.dataframe.DataFrame;

import java.io.PrintStream;
import java.util.List;
import java.util.Set;

public interface Query {
    Object getValue(Group path, Aggregation aggregation);

    Dimensions getDimensions();

    Aggregation[] getAggregations();

    List<Group> getGroups();

    Query drillDown(Object... columns);

    Query drillUp();

    Query slice(Object value);

    Query dice(Set<Object> valuesSets);

    Query pivot(Aggregation aggregation, Object[] values);

    Query pivot(Aggregation aggregation);

    Query collapse();

    Query order(Aggregation... aggregations);

    Query on(Cuboid... cuboids);

    Cuboid getCuboid();
   
    void setDice(Set<Object> valuesSets);

    AggregatesTableModel asTableModel();

    DataFrame asDataFrame(boolean includeIndex);

    void printResults(PrintStream out, String caption, boolean html);

    /**
     * Add a listener to the list that's notified each time a change to the query occurs.
     *
     * @param listener the QueryListener
     */
    void addQueryListener(QueryListener listener);

    /**
     * Add a listener to the list that's notified each time a change to the query occurs. The listener will
     * automatically be disposed of should no other object have a reference to it.
     *
     * @param listener the QueryListener
     */
    void addWeakQueryListener(QueryListener listener);

    /**
     * Remove a listener to the list that's notified each time a change to the query occurs.
     *
     * @param listener the QueryListener
     */
    void removeQueryListener(QueryListener listener);

    /**
     * Remove all listeners to the list that's notified each time a change to the query occurs.
     */
    void removeQueryListeners();

}
