/*
 * Copyright (c) 2014 Macrofocus GmbH. All Rights Reserved.
 */
package com.macrofocus.molap.aggregates.query;

import com.macrofocus.molap.aggregates.cube.FlyweightSet;

import java.util.Arrays;

public class Structure {
    private static final boolean FLYWEIGHT = false;

    private static final FlyweightSet<Structure> cache = FLYWEIGHT ? new FlyweightSet<Structure>() {
        @Override
        protected int hashCode(Structure obj) {
            return Arrays.hashCode((obj).path);
        }

        @Override
        protected boolean equals(Structure a, Structure b) {
            if (a == b) return true;

            // Probably incorrect - comparing Boolean[] arrays with Arrays.equals
            return Arrays.equals(a.path, b.path);

        }
    } : null;

    private static Structure get(Boolean... path) {
        if(FLYWEIGHT) {
            return cache.put(new Structure(path));
        } else {
            return new Structure(path);
        }
    }

    private static Structure getDrillDown(Boolean last, Boolean... path) {
        Boolean[] newPath = new Boolean[path.length + 1];
        System.arraycopy(path, 0, newPath, 0, path.length);
        newPath[newPath.length - 1] = last;
        return get(newPath);
    }

    private static Structure getDrillUp(Boolean... path) {
        Boolean[] newPath = new Boolean[path.length - 1];
        System.arraycopy(path, 0, newPath, 0, newPath.length);
        return get(newPath);
    }

    private final Boolean[] path;

    protected Structure(Boolean... path) {
        this.path = path;
    }

    public static Structure createRoot() {
        return get();
    }

    public static Structure create(Boolean... path) {
        return get(path);
    }

    public Structure drillUp() {
        if(this.path.length > 0) {
            return getDrillUp(this.path);
        } else {
            return null;
        }
    }

    public Structure drillDown(Boolean... columns) {
        Boolean[] path = new Boolean[this.path.length + columns.length];
        System.arraycopy(this.path, 0, path, 0, this.path.length);
        System.arraycopy(columns, 0, path, this.path.length, columns.length);
        return Structure.create(path);
    }

    public Structure drillDown(Boolean valueAt) {
        return getDrillDown(valueAt, this.path);
    }

    public Boolean getLast() {
        return path[path.length - 1];
    }

    public Structure changeLast(Boolean value) {
        return drillUp().drillDown(value);
    }

    public Boolean[] getPath() {
        return path;
    }

    @Override
    public boolean equals(Object o) {
        if(FLYWEIGHT) {
            return this == o;
        } else {
            if (this == o) return true;
            if (!(o instanceof Structure)) return false;

            Structure path1 = (Structure) o;

            // Probably incorrect - comparing Boolean[] arrays with Arrays.equals
            return Arrays.equals(path, path1.path);

        }
    }

    @Override
    public int hashCode() {
        if(FLYWEIGHT) {
            return super.hashCode();
        } else {
            return Arrays.hashCode(path);
        }
    }

    @Override
    public String toString() {
        return "Structure{" +
                "path=" + Arrays.toString(path) +
                '}';
    }

    public boolean startsWith(Structure structure) {
        if (this==structure)
            return true;
        if (structure==null)
            return false;

        int length = this.path.length;
        if (structure.path.length >= length)
            return false;

        for (int i=0; i<structure.path.length; i++) {
            Boolean o1 = path[i];
            Boolean o2 = structure.path[i];
            if (!(o1==null ? o2==null : o1.equals(o2)))
                return false;
        }

        return true;
    }

}
