/*
 * Copyright (c) 2014 Macrofocus GmbH. All Rights Reserved.
 */
package com.macrofocus.molap.dataframe;

/**
 * An event that characterizes a change in a data frame.
 *
 * @param <R> the type of row keys
 * @param <C> the type of column keys
 */
public class DataFrameEvent<R,C> {
    private final boolean structureChanged;
    private final Iterable<R> affectedRows;
    private final Iterable<C> affectedColumns;

    /**
     *
     * @param affectedRows the affected rows
     * @param affectedColumns the affected columns
     * @param structureChanged true if the rows or columns of the data frame have changed; false if only the data have change.
     */
    public DataFrameEvent(Iterable<R> affectedRows, Iterable<C> affectedColumns, boolean structureChanged) {
        this.affectedRows = affectedRows;
        this.affectedColumns = affectedColumns;
        this.structureChanged = structureChanged;
    }

    /**
     * Returns the rows that have been affected by the change.
     *
     * @return the modified rows
     */
    public Iterable<R> getAffectedRows() {
        return affectedRows;
    }

    /**
     * Rreturns the column that have been affected by teh change.
     *
     * @return the modified columns
     */
    public Iterable<C> getAffectedColumns() {
        return affectedColumns;
    }

    public boolean isStructureChanged() {
        return structureChanged;
    }
}
