/*
 * Copyright (c) 2014 Macrofocus GmbH. All Rights Reserved.
 */
package com.macrofocus.molap.dataframe;

import com.macrofocus.molap.series.Series;

import java.util.HashMap;
import java.util.Map;

@Deprecated
public class DataFrameStatistics {
    final DataFrameListener dataFrameListener = (DataFrameListener) new DataFrameListener() {
        public void dataFrameChanged(DataFrameEvent e) {
            if(e.getAffectedColumns() != null) {
                for (Object c : e.getAffectedColumns()) {
                    statistics.remove(c);
                }
            } else {
                statistics.clear();
            }
        }
    };

    public static final Method MIN = new Method() {
        public Number compute(DataFrame dataFrame, Object column, DataFrameStatistics statistics) {
            Number min = null;
            Series series = dataFrame.getColumn(column);
            for (Object row: dataFrame.rows()) {
                Object object = series.get(row);
                if (object != null && object instanceof Number) {
                    Number number = (Number) object;
                    if (number instanceof Double) {
                        double d = number.doubleValue();
                        if (!Double.isNaN(d) && !Double.isInfinite(d)) {
                            if (min == null || number.doubleValue() < min.doubleValue()) {
                                min = number;
                            }
                        }
                    } else if (number instanceof Float) {
                        float f = number.floatValue();
                        if (!Float.isNaN(f) && !Float.isInfinite(f)) {
                            if (min == null || number.doubleValue() < min.doubleValue()) {
                                min = number;
                            }
                        }
                    } else {
                        if (min == null || number.doubleValue() < min.doubleValue()) {
                            min = number;
                        }
                    }
                }
            }
            return min;
        }
    };
    public static final Method MAX = new Method() {
        public Number compute(DataFrame dataFrame, Object column, DataFrameStatistics statistics) {
            Number max = null;
            Series series = dataFrame.getColumn(column);
            for (Object row: dataFrame.rows()) {
                Object object = series.get(row);
                if (object != null && object instanceof Number) {
                    Number number = (Number) object;
                    if (number instanceof Double) {
                        double d = number.doubleValue();
                        if (!Double.isNaN(d) && !Double.isInfinite(d)) {
                            if (max == null || number.doubleValue() > max.doubleValue()) {
                                max = number;
                            }
                        }
                    } else if (number instanceof Float) {
                        float f = number.floatValue();
                        if (!Float.isNaN(f) && !Float.isInfinite(f)) {
                            if (max == null || number.doubleValue() > max.doubleValue()) {
                                max = number;
                            }
                        }
                    } else {
                        if (max == null || number.doubleValue() > max.doubleValue()) {
                            max = number;
                        }
                    }
                }
            }
            return max;
        }
    };

    private final Map<Object, Map<Method, Number>> statistics;

    private final DataFrame tableModel;

    public DataFrameStatistics(final DataFrame dataFrame) {
        this.tableModel = dataFrame;

        statistics = new HashMap<Object, Map<Method, Number>>();

        dataFrame.addDataFrameListener(dataFrameListener);
    }

    public Number getMinimum(Object column) {
        return getValue(column, MIN);
    }

    public Number getMaximum(Object column) {
        return getValue(column, MAX);
    }

    public Number getValue(Object column, Method method) {
        if (statistics.containsKey(column)) {
            Map<Method, Number> values = statistics.get(column);
            if (values.containsKey(method)) {
                return values.get(method);
            }
        }
        Number value = method.compute(tableModel, column, this);
        final Map<Method, Number> values;
        if (statistics.containsKey(column)) {
            values = statistics.get(column);
        } else {
            values = new HashMap<Method, Number>();
            statistics.put(column, values);
        }
        values.put(method, value);
        return value;
    }

    private interface Method {
        Number compute(DataFrame dataFrame, Object column, DataFrameStatistics statistics);
    }

}
