/*
 * Copyright (c) 2014 Macrofocus GmbH. All Rights Reserved.
 */
package com.macrofocus.molap.dataframe.swing;

import com.macrofocus.data.table.AbstractTableModel;
import com.macrofocus.molap.dataframe.DataFrame;
import com.macrofocus.molap.dataframe.DataFrameEvent;
import com.macrofocus.molap.dataframe.DataFrameListener;

public class DataFrameTableModel extends AbstractTableModel {
    private final DataFrame dataFrame;

    private final DataFrameListener dataFrameListener = new DataFrameListener() {
        @Override
        public void dataFrameChanged(DataFrameEvent event) {
            if(event.isStructureChanged()) {
                fireTableStructureChanged();
            } else {
                fireTableDataChanged();
            }
        }
    };

    public DataFrameTableModel(DataFrame dataFrame) {
        this.dataFrame = dataFrame;

        dataFrame.addWeakDataFrameListener(dataFrameListener);
    }

    @Override
    public String getColumnName(int column) {
        final Object columnKey = dataFrame.getColumnKey(column);
        if(columnKey != null) {
            return columnKey.toString();
        } else {
            return null;
        }
    }

    @Override
    public Class<?> getColumnClass(int columnIndex) {
        return dataFrame.getColumnClass(dataFrame.getColumnKey(columnIndex));
    }

    @Override
    public int getRowCount() {
        return dataFrame.getRowCount();
    }

    @Override
    public int getColumnCount() {
        return dataFrame.getColumnCount();
    }

    @Override
    public Object getValueAt(int rowIndex, int columnIndex) {
        return dataFrame.getValueAt(dataFrame.getRowKey(rowIndex), dataFrame.getColumnKey(columnIndex));
    }
}
