/*
 * Copyright (c) 2014 Macrofocus GmbH. All Rights Reserved.
 */
package com.macrofocus.molap.index;

import java.util.HashSet;
import java.util.Iterator;
import java.util.Set;

public abstract class AbstractIndex<K> implements Index<K> {
    @Override
    public Index<K> union(Index<K> other) {
        Set<K> keys = new HashSet<K>();
        for (K k : keys()) {
            keys.add(k);
        }
        for (K k : other.keys()) {
            keys.add(k);
        }

        K[] array = (K[])keys.toArray();
        return new DefaultIndex<K>(array);
    }

    @Override
    public Index<K> intersection(Index<K> other) {
        Set<K> clone = new HashSet<K>();
        for (K k : keys()) {
            clone.add(k);
        }

        Iterator<K> it = clone.iterator();
        while (it.hasNext()) {
            if (!other.contains(it.next())) {
                it.remove();
            }
        }

        K[] array = (K[])clone.toArray();
        return new DefaultIndex<K>(array);
    }

    @Override
    public Iterable<K> keys() {
        return new Iterable<K>() {
            @Override
            public Iterator<K> iterator() {
                return new Iterator<K>() {
                    int i = 0;

                    @Override
                    public boolean hasNext() {
                        return i < getSize();
                    }

                    @Override
                    public K next() {
                        return getKey(i++);
                    }

                    @Override
                    public void remove() {
                        throw new UnsupportedOperationException();
                    }
                };
            }
        };
    }
}
