/*
 * Copyright (c) 2014 Macrofocus GmbH. All Rights Reserved.
 */
package com.macrofocus.molap.index;

import java.util.*;

public class DefaultIndex<K> extends AbstractIndex<K> {
    private final Map<K, Integer> index;
    private final List<K> list;

    public DefaultIndex(K... keys) {
        index = new HashMap<K, Integer>();
        list = new ArrayList<K>();

        for (int i = 0; i < keys.length; i++) {
            K key = keys[i];
            if(!index.containsKey(key)) {
                index.put(key, i);
                list.add(key);
            } else {
                throw new IllegalStateException("Index not unique for value " + key + " (address=" + i + ")");
            }
        }
    }

    public DefaultIndex(Index<K> index, Comparator<K> comparator) {
        this.index = new HashMap<K, Integer>();
        list = new ArrayList<K>();

        for (K key : index.keys()) {
            this.index.put(key, index.getAddress(key));
            list.add(key);
        }

        Collections.sort(list, comparator);
    }

    private DefaultIndex(Map<K, Integer> index, List<K> list) {
        this.index = index;
        this.list = list;
    }

    @Override
    public boolean contains(K key) {
        return index.containsKey(key);
    }

    @Override
    public Integer getAddress(K key) {
        return index.get(key);
    }

    @Override
    public Iterable<K> keys() {
        return list;
    }

    @Override
    public K getKey(int i) {
        if(i >= 0 && i < list.size()) {
            return list.get(i);
        } else {
            return null;
        }
    }

    @Override
    public int getSize() {
        return list.size();
    }

    @Override
    public Index<K> head(int count) {
        HashMap<K, Integer> index = new HashMap<K, Integer>();
        List<K> list = new ArrayList<K>();

        for(int i = 0; i < count; i++) {
            K key = this.list.get(i);
            Integer loc = getAddress(key);
            index.put(key, loc);
            list.add(key);
        }

        return new DefaultIndex<K>(index, list);
    }

    @Override
    public Index<K> tail(int count) {
        HashMap<K, Integer> index = new HashMap<K, Integer>();
        List<K> list = new ArrayList<K>();

        for(int i = 0; i < count; i++) {
            K key = this.list.get(this.list.size() - 1 - i);
            Integer loc = getAddress(key);
            index.put(key, loc);
            list.add(key);
        }

        return new DefaultIndex<K>(index, list);
    }

    public DefaultIndex<K> remove(K... keys) {
        List<K> removed = new ArrayList<K>(list);
        removed.removeAll(Arrays.asList(keys));

        return new DefaultIndex<K>(index, removed);
    }
}
