/*
 * Copyright (c) 2014 Macrofocus GmbH. All Rights Reserved.
 */
package com.macrofocus.molap.index;

import java.util.Iterator;

public final class IntegerIndex extends AbstractIndex<Integer> {
    private final int from;
    private final int to;

    public IntegerIndex(int from, int to) {
        this.from = from;
        this.to = to;
    }

    @Override
    public Integer getAddress(Integer key) {
        int k = key;
        return contains(k)? k - from : null;
    }

    @Override
    public Index<Integer> head(int count) {
        return new IntegerIndex(from, from + count);
    }

    @Override
    public Index<Integer> tail(int count) {
        return new IntegerIndex(to - count, to);
    }

    @Override
    public boolean contains(Integer key) {
        return key >= from && key <= to;
    }

    public boolean contains(int key) {
        return key >= from && key <= to;
    }

    @Override
    public Integer getKey(int i) {
        return from + i;
    }

    @Override
    public Iterable<Integer> keys() {
        return new Iterable<Integer>() {
            @Override
            public Iterator<Integer> iterator() {
                return new Iterator<Integer>() {
                    int i = from;

                    @Override
                    public boolean hasNext() {
                        return i <= to;
                    }

                    @Override
                    public Integer next() {
                        return i++;
                    }

                    @Override
                    public void remove() {
                        throw new UnsupportedOperationException();
                    }
                };
            }
        };
    }

    @Override
    public int getSize() {
        return to - from + 1;
    }
}
