/*
 * Copyright (c) 2014 Macrofocus GmbH. All Rights Reserved.
 */
package com.macrofocus.molap.index;

import java.util.Arrays;

public class MultiKey implements Comparable {
    private final Object[] array;

    public MultiKey(Object[] array) {
        this.array = array;
    }

    public Object get(int i) {
        return array[i];
    }

    public int getSize() {
        return array.length;
    }

    @Override
    public boolean equals(Object o) {
        if (this == o) return true;
        if (o == null || getClass() != o.getClass()) return false;

        MultiKey multiKey = (MultiKey) o;

        // Probably incorrect - comparing Object[] arrays with Arrays.equals
        return Arrays.equals(array, multiKey.array);

    }

    @Override
    public int hashCode() {
        return array != null ? Arrays.hashCode(array) : 0;
    }

    @Override
    public int compareTo(Object o) {
        if(o instanceof MultiKey) {
            MultiKey a = this;
            MultiKey a2 = (MultiKey)o;
            if (a==a2)
                return 0;
            if (a==null || a2==null)
                return -1;

            int length = a.array.length;
            if (a2.array.length != length)
                return 1;

            for (int i=0; i<length; i++) {
                Comparable o1 = (Comparable)a.array[i];
                Comparable o2 = (Comparable)a2.array[i];
                if (!(o1==null || o2==null)) {
                    int c = o1.compareTo(o2);
                    if(c != 0) {
                        return c;
                    }
                }
            }

            return 0;
        } else {
            return -1;
        }
    }

    @Override
    public String toString() {
        return Arrays.toString(array);
    }
}
