/*
 * Copyright (c) 2014 Macrofocus GmbH. All Rights Reserved.
 */
package com.macrofocus.molap.index;

import java.util.Iterator;
import java.util.NavigableMap;
import java.util.TreeMap;

public class SortedIndex<K> extends AbstractIndex<K> {
    private final NavigableMap<K, Integer> index;

    public SortedIndex(K... keys) {
        index = new TreeMap<K, Integer>();

        for (int i = 0; i < keys.length; i++) {
            K key = keys[i];
            if(!index.containsKey(key)) {
                index.put(key, i);
            } else {
                throw new IllegalStateException("Index not unique");
            }
        }
    }

    private SortedIndex(NavigableMap<K, Integer> index) {
        this.index = index;
    }

    @Override
    public boolean contains(K key) {
        return index.containsKey(key);
    }

    @Override
    public Integer getAddress(K key) {
        return index.get(key);
    }

    @Override
    public Index<K> head(int count) {
        Iterator<K> iterator = index.navigableKeySet().iterator();
        K to = null;
        for(int i = 0; i < count; i++) {
            to = iterator.next();
        }
        return new SortedIndex<K>(index.headMap(to, true));
    }

    @Override
    public Index<K> tail(int count) {
        Iterator<K> iterator = index.navigableKeySet().descendingIterator();
        K from = null;
        for(int i = 0; i < count; i++) {
            from = iterator.next();
        }
        return new SortedIndex<K>(index.tailMap(from, true));
    }

//    public Indexer slice(K key) {
//        index.keySet()
//    }

    @Override
    public K getKey(int i) {
        return null;
    }

    @Override
    public int getSize() {
        return index.size();
    }
}
