/*
 * Copyright (c) 2014 Macrofocus GmbH. All Rights Reserved.
 */
package com.macrofocus.molap.series;

import java.util.HashSet;
import java.util.Iterator;
import java.util.Objects;
import java.util.Set;

public abstract class AbstractSeries<K,V> implements Series<K,V> {
    @Override
    public Series<K, Double> pow(Double exponent) {
        Set<K> keys = new HashSet<K>();
        for (K k : keys()) {
            keys.add(k);
        }

        K[] array = (K[])keys.toArray();
        Double[] values = new Double[keys.size()];

        for (int i = 0; i < array.length; i++) {
            K k = array[i];
            V v1 = get(k);
            if(v1 instanceof Number) {
                values[i] = Math.pow(((Number) v1).doubleValue(), exponent);
            }
        }

        return new IndexedSeries<K, Double>(getName() + "^" + exponent, values, array);
    }

    @Override
    public Series<K, Double> divide(Series<K, V> other) {
        Set<K> keys = new HashSet<K>();
        for (K k : keys()) {
            keys.add(k);
        }
        for (K k : other.keys()) {
            keys.add(k);
        }

        K[] array = (K[])keys.toArray();
        Double[] values = new Double[keys.size()];

        for (int i = 0; i < array.length; i++) {
            K k = array[i];
            V v1 = get(k);
            V v2 = other.get(k);
            if(v1 instanceof Number && v2 instanceof Number) {
                values[i] = ((Number)v1).doubleValue() / ((Number)v2).doubleValue();
            }
        }

        return new IndexedSeries<K, Double>(getName() + "/" + other.getName(), values, array);
    }

    @Override
    public Series<K, Double> multiply(Series<K, V> other) {
        Set<K> keys = new HashSet<K>();
        for (K k : keys()) {
            keys.add(k);
        }
        for (K k : other.keys()) {
            keys.add(k);
        }

        K[] array = (K[])keys.toArray();
        Double[] values = new Double[keys.size()];

        for (int i = 0; i < array.length; i++) {
            K k = array[i];
            V v1 = get(k);
            V v2 = other.get(k);
            if(v1 instanceof Number && v2 instanceof Number) {
                values[i] = ((Number)v1).doubleValue() * ((Number)v2).doubleValue();
            }
        }

        return new IndexedSeries<K, Double>(getName() + "*" + other.getName(), values, array);
    }


    @Override
    public Series<K, Double> subtract(Series<K, V> other) {
        Set<K> keys = new HashSet<K>();
        for (K k : keys()) {
            keys.add(k);
        }
        for (K k : other.keys()) {
            keys.add(k);
        }

        K[] array = (K[])keys.toArray();
        Double[] values = new Double[keys.size()];

        for (int i = 0; i < array.length; i++) {
            K k = array[i];
            V v1 = get(k);
            V v2 = other.get(k);
            if(v1 instanceof Number && v2 instanceof Number) {
                values[i] = ((Number)v1).doubleValue() - ((Number)v2).doubleValue();
            }
        }

        return new IndexedSeries<K, Double>(getName() + "-" + other.getName(), values, array);
    }

    @Override
    public Series<K, Double> add(Series<K, V> other) {
        Set<K> keys = new HashSet<K>();
        for (K k : keys()) {
            keys.add(k);
        }
        for (K k : other.keys()) {
            keys.add(k);
        }

        K[] array = (K[])keys.toArray();
        Double[] values = new Double[keys.size()];

        for (int i = 0; i < array.length; i++) {
            K k = array[i];
            V v1 = get(k);
            V v2 = other.get(k);
            if(v1 instanceof Number && v2 instanceof Number) {
                values[i] = ((Number)v1).doubleValue() + ((Number)v2).doubleValue();
            }
        }

        return new IndexedSeries<K, Double>(getName() + "+" + other.getName(), values, array);
    }

    @Override
    public K indexMax() {
        return null;
    }

    @Override
    public K indexMin() {
        return null;
    }

    @Override
    public Integer countUnique() {
        return null;
    }

    @Override
    public V max() {
        return null;
    }

    @Override
    public V min() {
        return null;
    }

    @Override
    public V median() {
        return null;
    }

    @Override
    public Double meanAbsoluteDeviation() {
        return null;
    }

    @Override
    public Double mean() {
        Integer count = count();
        Double sum = sum();
        if(count != null && count > 0) {
            return sum / count;
        } else {
            return null;
        }
    }

    @Override
    public Integer count() {
        int count = 0;
        for (K k : keys()) {
            V value = get(k);
            if(value != null) {
                count++;
            }
        }
        return count;
    }

    @Override
    public Double sum() {
        double sum = 0;
        for (K k : keys()) {
            V value = get(k);
            if(value != null && value instanceof Number) {
                sum += ((Number)value).doubleValue();
            }
        }
        return sum;
    }

    @Override
    public void print() {
        for (K k : keys()) {
            V value = get(k);
            System.out.println(k + "\t" + value);
        }
        System.out.println("Name: " + getName() + ", type: " + getType());
    }

    @Override
    public String toString() {
        return getName().toString();
    }

    @Override
    public boolean equals(Object o) {
        if (this == o) return true;
        if (!(o instanceof Series)) return false;

        Series that = (Series) o;

        return getName().equals(that.getName());

    }

//    @Override
    public boolean identitcal(Series o) {
        if (this == o) return true;
        if (!(o instanceof Series)) return false;

        Series that = (Series) o;

        if (!getName().equals(that.getName())) return false;
        if (!getType().equals(that.getType())) return false;

        Iterator iterator1 = keys().iterator();
        Iterator iterator2 = that.keys().iterator();

        while (iterator1.hasNext()) {
            if (!iterator2.hasNext()) {
                return false;
            }
            Object o1 = iterator1.next();
            Object o2 = iterator2.next();
            if (!Objects.equals(o1, o2)) {
                return false;
            }
        }
        return !iterator2.hasNext();
    }

    @Override
    public int hashCode() {
        assert getName() != null: getClass();

        return getName().hashCode();
    }
}
