/*
 * Copyright (c) 2014 Macrofocus GmbH. All Rights Reserved.
 */
package com.macrofocus.molap.series;

import com.macrofocus.molap.index.Index;
import com.macrofocus.molap.index.IntegerIndex;

public class DefaultSeries<V> extends AbstractSeries<Integer,V> implements MutableSeries<Integer,V> {
    private final Object name;
    private final V[] values;
    private final Index<Integer> index;

    public DefaultSeries(V[] values) {
        this.name = null;
        this.values = values;
        this.index = new IntegerIndex(0, values.length - 1);
    }

    public DefaultSeries(Object name, V[] values) {
        this.name = name;
        this.values = values;
        this.index = new IntegerIndex(0, values.length - 1);
    }

    private DefaultSeries(V[] values, Index<Integer> index) {
        this.name = null;
        this.values = values;
        this.index = index;
    }

    @Override
    public Object getName() {
        return name;
    }

    @Override
    public Class getType() {
        return values.getClass().getComponentType();
    }

    @Override
    public V get(Integer key) {
        return values[key];
    }

    @Override
    public Integer getKey(int i) {
        return index.getKey(i);
    }

    @Override
    public void set(Integer key, V value) {
        values[key] = value;
    }

    @Override
    public Iterable<Integer> keys() {
        return index.keys();
    }

    @Override
    public <L> Series<L, V> reindex(L... keys) {
        return null;
    }

    @Override
    public Series<Integer, V> head(int count) {
        return reindex(index.head(count));
    }

    @Override
    public Series<Integer, V> tail(int count) {
        return reindex(index.tail(count));
    }

    public <K> Series<K, V> reindex(Index<K> index) {
        return new IndexedSeries<K,V>(getName(), values, index);
    }
}
