/*
 * Copyright (c) 2014 Macrofocus GmbH. All Rights Reserved.
 */
package com.macrofocus.molap.series;

import com.macrofocus.molap.index.DefaultIndex;
import com.macrofocus.molap.index.Index;

public class IndexedSeries<K,V> extends AbstractSeries<K,V> {
    private final V[] values;
    private final Index<K> index;
    private final Object name;

    public IndexedSeries(Object name, V[] values, K[] keys) {
        this.name = name;
        this.values = values;

        index = new DefaultIndex<K>(keys);
    }

    public IndexedSeries(Object name, V[] values, Index<K> index) {
        this.name = name;
        this.values = values;

        this.index = index;
    }

    @Override
    public Object getName() {
        return name;
    }

    @Override
    public Class getType() {
        return values.getClass().getComponentType();
    }

    @Override
    public V get(K key) {
        final Integer index = this.index.getAddress(key);
        if(index != null) {
            return values[index];
        } else {
            return null;
        }
    }

    @Override
    public K getKey(int i) {
        return index.getKey(i);
    }

    @Override
    public Iterable<K> keys() {
        return index.keys();
    }

    @Override
    public <L> Series<L, V> reindex(L... keys) {
        return null;
    }

    @Override
    public Series<K, V> head(int count) {
        return new IndexedSeries<K, V>("head(" + getName() + ")", values, index.head(count));
    }

    @Override
    public Series<K, V> tail(int count) {
        return new IndexedSeries<K, V>("tail(" + getName() + ")", values, index.tail(count));
    }}
