/*
 * Copyright (c) 2014 Macrofocus GmbH. All Rights Reserved.
 */
package com.macrofocus.molap.table;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

public abstract class AbstractIndexedTable extends AbstractTable implements IndexedTable {
    private final Map<Columns,Map<ColumnsValue, Row>> uniqueIndices;
    private final Map<Columns,Map<ColumnsValue, List<Row>>> indices;

    protected AbstractIndexedTable() {
        uniqueIndices = new HashMap<Columns,Map<ColumnsValue, Row>>();
        indices = new HashMap<Columns, Map<ColumnsValue, List<Row>>>();
    }

    @Override
    public Map<ColumnsValue, Row> getUniqueIndex(Columns key) {
        if(!uniqueIndices.containsKey(key)) {
            Map<ColumnsValue, Row> index = new HashMap<ColumnsValue, Row>();
            for(int r = 0; r < getRowCount(); r++) {
                ColumnsValue value = key.getValue(r);

                assert !index.containsKey(value): "Index is not unique!";

                index.put(value, getRow(r));
            }
            uniqueIndices.put(key, index);
        }
        return uniqueIndices.get(key);
    }

    @Override
    public Map<ColumnsValue, List<Row>> getIndex(Columns key) {
        if(!indices.containsKey(key)) {
            Map<ColumnsValue, List<Row>> index = new HashMap<ColumnsValue, List<Row>>();
            for(int r = 0; r < getRowCount(); r++) {
                ColumnsValue value = key.getValue(r);

                if(!index.containsKey(value)) {
                    index.put(value, new ArrayList<Row>());
                }
                index.get(key).add(getRow(r));
            }
            indices.put(key, index);
        }
        return indices.get(key);
    }
}
