/*
 * Copyright (c) 2014 Macrofocus GmbH. All Rights Reserved.
 */
package com.macrofocus.molap.table;

import com.macrofocus.molap.aggregates.cube.FlyweightSet;

import java.io.PrintStream;

public abstract class AbstractTable implements Table {
    private static final boolean FLYWEIGHT = false;
    private static final FlyweightSet<TableColumn> columns = FLYWEIGHT ? new FlyweightSet<TableColumn>() : null;
    private static final FlyweightSet<Row> rows = FLYWEIGHT ? new FlyweightSet<Row>() : null;

    @Override
    public TableColumn getColumn(int column) {
        final TableColumn c = new TableColumn(this, column);
        if(FLYWEIGHT) {
            return columns.put(c);
        } else {
            return c;
        }
    }

    @Override
    public TableColumn findColumn(String name) {
        int column = -1;
        for (int c = 0; c < getColumnCount(); c++) {
            if (name.equals(getColumnName(c))) {
                column = c;
                break;
            }
        }

        assert column >= 0: "Could not find column named " + name;

        return getColumn(column);
    }

    @Override
    public Row getRow(int row) {
        final Row r = new Row(this, row);
        if(FLYWEIGHT) {
            return rows.put(r);
        } else {
            return r;
        }
    }

    @Override
    public void print(PrintStream out, String caption, boolean html) {
        if(!html) {
            if(caption != null) {
                System.out.println(caption);
            }

            int cols = getColumnCount();
            for (int i = 0; i < cols; i++) {
                final String o = getColumnName(i);
                final String s;
                if(o != null) {
                    s = o;
                } else {
                    s = "n.a.";
                }
                out.printf("%20s", abbreviate(s, 19));
            }
            //        System.out.println();
            //        for (int i = 0; i < cols; i++) {
            //            final String o = tableModel.getColumnClass(i).getSimpleName();
            //            final String s;
            //            if(o != null) {
            //                s = o;
            //            } else {
            //                s = "n.a.";
            //            }
            //            out.printf("%20s", abbreviate(s, 19));
            //        }
            out.println();
            for(int row = 0; row < getRowCount(); row++) {
                for (int i = 0; i < cols; i++) {
                    final Object o = getValueAt(row, i);
                    final String s;
                    if(o != null) {
                        s = o.toString();
                    } else {
                        s = "n.a.";
                    }
                    out.printf("%20s", abbreviate(s, 19));
                }
                out.println();
            }
        } else {
            out.println("<table style=\"width: auto; border-collapse: separate;\">");

            if(caption != null) {
                out.println("<caption>" + caption + "</caption>");
            }

            out.println("<tbody>");
            out.println("<tr>");
            int cols = getColumnCount();
            for (int i = 0; i < cols; i++) {
                final String o = getColumnName(i);
                final String s;
                if(o != null) {
                    s = o;
                } else {
                    s = "n.a.";
                }
                out.print("<th>");
                out.print(s);
                out.println("</th>");
            }
            out.println("</tr>");

            for(int row = 0; row < getRowCount(); row++) {
                out.println("<tr>");
                for (int i = 0; i < cols; i++) {
                    final Object o = getValueAt(row, i);
                    final String s;
                    if(o != null) {
                        s = o.toString();
                    } else {
                        s = "n.a.";
                    }
                    if(o instanceof Number) {
                        out.print("<td align=\"right\">");
                    } else {
                        out.print("<td>");
                    }
                    out.print(s);
                    out.println("</td>");
                }
                out.println("</tr>");
            }
            out.println("</tbody>");

            out.println("</table>");
        }
    }

    private static String abbreviate(String str, int maxWidth) {
        if(str == null) {
            return null;
        }
        if (str.length() <= maxWidth) {
            return str;
        }
        return str.substring(0, maxWidth - 3) + "...";
    }
}
