/*
 * Copyright (c) 2014 Macrofocus GmbH. All Rights Reserved.
 */
package com.macrofocus.molap.table;

import java.util.ArrayList;
import java.util.List;

public class ColumnsTable extends AbstractTable {
    private final List<Column> columns;

    public ColumnsTable(Table table) {
        columns = new ArrayList<Column>();
        for(int c = 0; c < table.getColumnCount(); c++) {
            columns.add(table.getColumn(c));
        }
    }

    public void addColumns(Column... columns) {
        for (Column column : columns) {
            this.columns.add(column);
        }
    }

    @Override
    public int getRowCount() {
        return columns.get(0).getRowCount();
    }

    @Override
    public int getColumnCount() {
        return columns.size();
    }

    @Override
    public String getColumnName(int columnIndex) {
        return columns.get(columnIndex).getName();
    }

    @Override
    public Class<?> getColumnClass(int columnIndex) {
        return columns.get(columnIndex).getType();
    }

    @Override
    public Object getValueAt(int rowIndex, int columnIndex) {
        return columns.get(columnIndex).getValueAt(rowIndex);
    }
}
