/*
 * Copyright (c) 2014 Macrofocus GmbH. All Rights Reserved.
 */
package com.macrofocus.molap.table;

import java.util.HashMap;
import java.util.Map;

public class LookupColumn extends AbstractColumn {
    private final Columns primaryKey;
    private final Columns foreignKey;
    private final Column value;

    private final Map<ColumnsValue, Row> index;

    public LookupColumn(Columns primaryKey, Columns foreignKey, Column value) {
        this.primaryKey = primaryKey;
        this.foreignKey = foreignKey;
        this.value = value;

        index = new HashMap<ColumnsValue, Row>();
        for(int r = 0; r < primaryKey.getRowCount(); r++) {
            ColumnsValue columnsValue = primaryKey.getValue(r);

            assert !index.containsKey(columnsValue): "Index is not unique!";

            index.put(columnsValue, primaryKey.getRow(r));
        }
    }

    @Override
    public String getName() {
        return value.getName();
    }

    @Override
    public Class getType() {
        return value.getType();
    }

    @Override
    public int getRowCount() {
        return foreignKey.getRowCount();
    }

    @Override
    public Row getRow(int row) {
        return foreignKey.getRow(row);
    }

    @Override
    public Object getValueAt(int row) {
        final Row lookUp = index.get(foreignKey.getValue(row));
        if(lookUp != null) {
            return value.getValueAt(lookUp.getRow());
        } else {
            return null;
        }
    }
}
