/*
 * Copyright (c) 2014 Macrofocus GmbH. All Rights Reserved.
 */
package com.macrofocus.molap.table;

public class Row implements Comparable {
    private final Table table;
    private final int row;

    public Row(Table table, int row) {
        this.table = table;
        this.row = row;
    }

    public boolean isValid() {
        return row >= 0;
    }

    public Object getValueAt(int column) {
        return table.getValueAt(row, column);
    }

    public Table getTable() {
        return table;
    }

    public int getRow() {
        return row;
    }

    @Override
    public boolean equals(Object o) {
        if (this == o) return true;
        if (o == null || getClass() != o.getClass()) return false;

        Row row1 = (Row) o;

        assert table.equals(row1.table) : table + " != " + row1.table;

        if (row != row1.row) return false;
        return !(table != null ? !table.equals(row1.table) : row1.table != null);

    }

    @Override
    public int hashCode() {
        int result = table != null ? table.hashCode() : 0;
        result = 31 * result + row;
        return result;
    }

    @Override
    public int compareTo(Object o) {
        Row row1 = (Row) o;

        if (table != null && !table.equals(row1.table)) {
            return (hashCode()< table.hashCode() ? -1 : (hashCode()== table.hashCode() ? 0 : 1));
        }

        return (row<row1.row ? -1 : (row==row1.row ? 0 : 1));
    }

    @Override
    public String toString() {
        return "Row{" +
                "factTable=" + table +
                ", row=" + row +
                '}';
    }
}

