/*
 * Copyright (c) 2014 Macrofocus GmbH. All Rights Reserved.
 */
package com.macrofocus.molap.table;

public class TableColumn extends AbstractColumn {
    private final Table table;
    private final int column;
    private final String name;

    public TableColumn(Table table, int column) {
        this.table = table;
        this.column = column;
        this.name = null;
    }

    @Override
    public String getName() {
        if (name != null) {
            return name;
        } else {
            return table.getColumnName(column);
        }
    }

    @Override
    public Class getType() {
        return table.getColumnClass(column);
    }

    @Override
    public int getRowCount() {
        return table.getRowCount();
    }

    @Override
    public Row getRow(int row) {
        return table.getRow(row);
    }

    @Override
    public Object getValueAt(int row) {
        return table.getValueAt(row, column);
    }

    @Override
    public boolean equals(Object o) {
        if (this == o) return true;
        if (o == null || getClass() != o.getClass()) return false;

        TableColumn column1 = (TableColumn) o;

        assert table.equals(column1.table) : table + " != " + column1.table;

        if (column != column1.column) return false;
        return !(table != null ? !table.equals(column1.table) : column1.table != null);

    }

    @Override
    public int hashCode() {
        int result = table != null ? table.hashCode() : 0;
        result = 31 * result + column;
        return result;
    }

    @Override
    public String toString() {
        return getName();
    }
}