/*
 * Copyright (c) 2014 Macrofocus GmbH. All Rights Reserved.
 */
package com.macrofocus.molap.table;

import java.util.*;

public class UnpivotTable extends AbstractTable {
    private final Table table;
    private final String columnName;
    private final String valueName;
    private final Column[] unpivotColumns;
    private final List<Column> columns;

    private final Class commonColumnType;
    private Class commonValueType;

    public UnpivotTable(Table table, String columnName, String valueName, Column... unpivotColumns) {
        this.table = table;
        this.columnName = columnName;
        this.valueName = valueName;
        this.unpivotColumns = unpivotColumns;

        Set<Column> set = new HashSet<Column>();
        set.addAll(Arrays.asList(unpivotColumns));

        this.columns = new ArrayList<Column>();

        for(int c = 0; c < table.getColumnCount(); c++) {
            Column column = table.getColumn(c);
            if(!set.contains(column)) {
                this.columns.add(new OriginalColumn(column));
            }
        }

        commonColumnType = String.class;

        for (Column column : unpivotColumns) {
            if(commonValueType == null) {
                commonValueType = column.getType();
            } else {
                if(commonValueType != column.getType()) {
                    commonValueType = Object.class;
                }
            }
        }

        this.columns.add(new UnipivotColumn());
        this.columns.add(new UnipivotValue());
    }

    @Override
    public int getRowCount() {
        return table.getRowCount() * unpivotColumns.length;
    }

    @Override
    public int getColumnCount() {
        return columns.size();
    }

    @Override
    public String getColumnName(int columnIndex) {
        return columns.get(columnIndex).getName();
    }

    @Override
    public Class<?> getColumnClass(int columnIndex) {
        return columns.get(columnIndex).getType();
    }

    @Override
    public Object getValueAt(int rowIndex, int columnIndex) {
        return columns.get(columnIndex).getValueAt(rowIndex);
    }

    private class OriginalColumn extends AbstractColumn {
        private final Column column;

        private OriginalColumn(Column column) {
            this.column = column;
        }

        @Override
        public String getName() {
            return column.getName();
        }

        @Override
        public Class getType() {
            return column.getType();
        }

        @Override
        public int getRowCount() {
            return column.getRowCount() * unpivotColumns.length;
        }

        @Override
        public Row getRow(int row) {
            return null;
        }

        @Override
        public Object getValueAt(int row) {
            return column.getValueAt(row / unpivotColumns.length);
        }
    }

    private class UnipivotColumn extends AbstractColumn {
        @Override
        public String getName() {
            return columnName;
        }

        @Override
        public Class getType() {
            return String.class;
        }

        @Override
        public int getRowCount() {
            return table.getRowCount() * unpivotColumns.length;
        }

        @Override
        public Row getRow(int row) {
            return null;
        }

        @Override
        public Object getValueAt(int row) {
            return unpivotColumns[row % unpivotColumns.length].getName();
        }
    }

    private class UnipivotValue extends AbstractColumn {
        @Override
        public String getName() {
            return valueName;
        }

        @Override
        public Class getType() {
            return commonValueType;
        }

        @Override
        public int getRowCount() {
            return table.getRowCount() * unpivotColumns.length;
        }

        @Override
        public Row getRow(int row) {
            return null;
        }

        @Override
        public Object getValueAt(int row) {
            return unpivotColumns[row % unpivotColumns.length].getValueAt(row / unpivotColumns.length);
        }
    }
}
